//******************************************************************
//* ANAHEIM AUTOMATION BLDC MOTOR DRIVER ROUTINES
//* WRITTEN BY FRED EADY FOR SERVO MAGAZINE
//* LAST UPDATED 11/18/2008
//* CHANGES/ADDITIONS
//******************************************************************
#include <pic18.h>
#include <stdio.h>
#include <ctype.h>
#include <htc.h>

// Configuration Fuse Settings for PIC18F2431
__CONFIG(1,IESODIS & FCMDIS & HSPLL);
__CONFIG(2,BORDIS & PWRTDIS & WDTDIS);
__CONFIG(3,PWMOUTEN & LOWPOLLO & HIPOLLO & MCLREN);
__CONFIG(4,DEBUGEN & LVPDIS & STVREN);
__CONFIG(5,UNPROTECT);
__CONFIG(6,WRTEN);
__IDLOC(1117);

//******************************************************************
//*	FUNCTION PROTOTYPES
//******************************************************************
void init(void);
void pwm_control(char cmd);

//******************************************************************
//*	GLOBAL VARIABLES
//******************************************************************
char x,isecs1,tsecs1;
unsigned int tmsecs1,imsecs1;

//******************************************************************
//*	GLOBAL DEFINITIONS
//******************************************************************
#define	MOTOR_I	RA0
#define	SPD_REF	RA1
#define	PHASE_A	RA2
#define PHASE_B	RA3
#define	PHASE_C	RA4

#define	SW1		RC0
#define FAULT_I	RC1
#define SW2		RC2
#define	LED1	LATC3
#define LED2	LATC4
#define	actled	LATC5
#define	OFF		0
#define ON		1
				
#define	UPDATE_EN	UDIS = 0
#define UPDATE_DIS	UDIS = 1
//******************************************************************
//*	INTERRUPT DEFINITIONS
//******************************************************************
#define		disable_TMR2int  	TMR2IE = 0
#define		enable_TMR2int	 	TMR2IE = 1
#define		disable_TMR1int  	TMR1IE = 0
#define		enable_TMR1int	 	TMR1IE = 1
#define		disable_TMR0int  	TMR0IE = 0
#define		enable_TMR0int	 	TMR0IE = 1

#define		enable_GLOBALint  	IPEN = 0;				\
							  	GIE = 1;				\
							  	PEIE = 1;
#define		disable_GLOBALint 	IPEN = 0;				\
							  	GIE = 0;				\
							  	PEIE = 0;
#define		TIMER0ON			TMR0ON = 1
#define		TIMER0OFF			TMR0ON = 0
#define		TIMER1ON			TMR1ON = 1
#define		TIMER1OFF			TMR1ON = 0
#define		TIMER2ON		  	TMR2ON = 1
#define		TIMER2OFF		  	TMR2ON = 0

#define mdelay1(msecdelay)		TIMER1OFF;					\
								TMR1IF = 0;					\
								imsecs1 = 0;				\
								tmsecs1 = 0;				\
								TIMER1ON;					\
								while(tmsecs1 < msecdelay);
								
#define	sdelay1(secdelay)		TIMER1OFF;					\
								TMR1IF = 0;					\
								imsecs1 = 0;				\
								tmsecs1 = 0;				\
								isecs1 = 0;					\
								tsecs1 = 0;					\
								TIMER1ON;					\
								while(tsecs1 < secdelay);

//******************************************************************
//*	INTERRUPT HANDLER ROUTINE
//******************************************************************
void interrupt TIMERS(void)
{
	if((TMR1IF && TMR1IE))
	{
		TMR1IF = 0;
		TMR1H = 0xEC;	
		TMR1L = 0x77;

		++tmsecs1;
		if(++imsecs1 == 1000)
		{
			imsecs1 = 0;
			++isecs1;
			++tsecs1;
			actled ^= 1;
			//++PDC2L;
			//if(PDC2L == 0)
			//	++PDC2H;
		}
	}
}
//******************************************************************
//*	INITIALIZE ROUTINE
//******************************************************************
void init(void)
{
   	TRISA = 0b11111111;		
   	//TRISB = 0b11000000;		
   	TRISC = 0b11000111;
//******************************************************************
//*	CONFIGURE EEPROM READ/WRITE
//******************************************************************
	EECON1 = 0b00000000;
//******************************************************************
//*	CONFIGURE A2D 
//******************************************************************
	ADCON0 = 0b00000000;	//turn OFF A2D
	ADCON1 = 0b00000000;	//
	ADCON2 = 0b00000000;
//******************************************************************
//*	CONFIGURE AND START TIMER1
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	TIMER1OFF;
	T1CON = 0b00000000;
	TMR1H = 0xEC;
	TMR1L = 0x77;
	TIMER1ON;
//******************************************************************
//*	CONFIGURE PWM 
//******************************************************************
	FLTCONFIG 	= 0b00000000;	//set bit 7 when debugging
	PTCON0		= 0b00000000;
	PTCON1		= 0b00000000;
	PWMCON1 	= 0b00000000;
	PTPERH		= 0x00;
	PTPERL		= 0xFA;
	pwm_control(ON);
//******************************************************************
//*	ACTIVATE INTERRUPTS
//******************************************************************
	enable_TMR1int;
    enable_GLOBALint;
}
//******************************************************************
//*	PWM CONTROL
//******************************************************************
void pwm_control(char cmd)
{
	switch (cmd)
	{
		case ON:
			PWMCON0 = 0b01011110;		
			PTEN = 1;
		break;
		case OFF:
			PTEN = 0;				
			PWMCON0 = 0b00001110;	
			PORTB = 0xFF;
		break;
	}
}

void main(void)
{
//******************************************************************
//*	INITIALIZE
//******************************************************************

	
  	init();				//
  	//UPDATE_DIS;
  	PDC0H = 0x01;
  	PDC0L = 0xFF;
  	PDC1H = 0x01;
  	PDC1L = 0xFF;
  	PDC2H = 0x01;
  	PDC2L = 0xFF;
  	OVDCOND = 0b00100000;
  	OVDCONS = 0b00000000;
  	//UPDATE_EN;
  	LED1 = OFF;
  	LED2 = OFF;
  	do{
	  	++x;

	}while(1);
 }

